// vertex shader to fragment shader
varying vec3 vNormal;   // view space normal
varying vec3 vPos;      // view-space coordinate of the vertex
varying vec3 wPos;      // world-space coordinate of the vertex
#ifdef ENABLE_TEXTURING
	varying vec2 vTexCoord;
#endif

// scene parameters
uniform vec4 ambient;

// the surface parameters of the mesh
uniform float Kd;
uniform float Ks;
uniform float Ka;
uniform float n_specular;

#ifdef ENABLE_TEXTURING
	uniform sampler2D texSource;
#endif

vec4 vDiffuse;
vec4 vSpecular;

void main(void) {   
	vec3 vNormal_N = normalize(vNormal);
	
	// Compute the light direction in world space:
	vec3 vLightDiff = re_SpotLights[0].position-wPos;
	float vLightDistance = length(vLightDiff);
	vec3 vLightDir_N = normalize(vec3(re_ViewMatrix * vec4(vLightDiff / vLightDistance,0)));
	
	// Compute reflection vector in view space:
	vec3 vReflect = normalize( 2.0 * dot(vNormal_N, vLightDir_N) * vNormal_N - vLightDir_N );         

	// Compute view vector (view space):
	vec3 vView = -normalize(vPos);                

	// Compute ambient contribution: 
	vDiffuse = vec4(0.0);
	vSpecular = vec4(0.0);
	
	// the dot product between vertex normal and light direction (view space)
	float NdotL = max(dot(vNormal_N,vLightDir_N),0.0);
	float test = 0.0;	
	if(NdotL > 0.0) {
		// convert spot light direction into view space
		vec3 vSpotDirNorm = normalize ( vec3( re_ViewMatrix * vec4(-re_SpotLights[0].direction, 0) ) );
		float spotEffect = dot(vSpotDirNorm, vLightDir_N);
		//test = spotEffect;
		
		if (spotEffect > re_SpotLights[0].attenDir.x) {
			// Compute diffuse contribution: 
			vDiffuse += re_SpotLights[0].color * Kd * NdotL;
			
			// Compute specular term:
			vSpecular = re_SpotLights[0].color * Ks * pow(max(0.0, dot(vReflect, vView)), n_specular);

			// apply attenuation
			float attenuation = 1.0 / (
				re_SpotLights[0].attenPos.x +         // constant
				re_SpotLights[0].attenPos.y * vLightDistance +     // linear
				re_SpotLights[0].attenPos.z * vLightDistance * vLightDistance); // quadric
			vDiffuse = vDiffuse * attenuation ; 

		      //calculate fallOff
		      float spotLimit = re_SpotLights[0].attenDir.x;
		      float fallOffLimit = (100.0+re_SpotLights[0].attenDir.y)* re_SpotLights[0].attenDir.x/100.0;
		      float mix = 1.0;
		      if (  spotEffect < fallOffLimit){
		      mix = (spotEffect - spotLimit)/(fallOffLimit -spotLimit);
		      }
		      vDiffuse = vDiffuse * mix;
		      vSpecular = vSpecular*mix;

		}



	}

#ifdef ENABLE_TEXTURING
	gl_FragColor = texture2D(texSource, vTexCoord ) * (vDiffuse + ambient * Ka) ;
#else	
	gl_FragColor = (vDiffuse + ambient * Ka) ;
#endif
	gl_FragColor += vSpecular;
}
