
inline Real Substance::getMass() const { 
	return mOdeMass.mass;
}
inline REng::Vector3 Substance::getCenterOfGravity() const{
	return REng::Vector3(
		SIM_TO_WORLD(mOdeMass.c[0]), 
		SIM_TO_WORLD(mOdeMass.c[1]), 
		SIM_TO_WORLD(mOdeMass.c[2]));
}
inline REng::Matrix3 Substance::getInertiaTensor() const{
	return REng::Matrix3(
		mOdeMass.I[0],mOdeMass.I[1],mOdeMass.I[2],
		mOdeMass.I[4],mOdeMass.I[5],mOdeMass.I[6],
		mOdeMass.I[8],mOdeMass.I[9],mOdeMass.I[10]
		);
}

inline void Substance::synchAuto(){
	if(mAutoSynch) synch();
}

inline bool Substance::isValid() const{
	return dMassCheck(&mOdeMass)!=0;
}
inline void Substance::setZero(){
	mOdeMass.setZero(); 
	synchAuto();
}
inline void Substance::setAll(Real mass, const REng::Vector3& cog, Real I11, Real I22, Real I33, Real I12, Real I13, Real I23){
	mOdeMass.setParameters(
		mass,
		WORLD_TO_SIM(cog[0]),WORLD_TO_SIM(cog[1]),WORLD_TO_SIM(cog[2]),
		I11, I22, I33, I12, I13, I23
	); 
	synchAuto();
}
inline void Substance::setMassAndAdjust(Real mass){
	mOdeMass.adjust(mass); 
	synchAuto();
}
inline void Substance::add(const Substance& prop){
	mOdeMass.add(&prop.mOdeMass);
	synchAuto();
}
inline void Substance::setSphere(Real density, Real radius){
	mOdeMass.setSphere(density, WORLD_TO_SIM(radius));
	synchAuto();
}
inline void Substance::setCapsule(Real density, CapsuleOrientation direction, Real radius, Real length){
	mOdeMass.setCapsule(density, direction, WORLD_TO_SIM(radius), WORLD_TO_SIM(length));
	synchAuto();
}
inline void Substance::setCylinder(Real density, CapsuleOrientation direction, Real radius, Real length){
	mOdeMass.setCylinder(density, direction, WORLD_TO_SIM(radius), WORLD_TO_SIM(length));
	synchAuto();
}
inline void Substance::setBox(Real density, const REng::Vector3& size){
	mOdeMass.setBox(density, 
		WORLD_TO_SIM(size[0]), WORLD_TO_SIM(size[1]), WORLD_TO_SIM(size[2]));
	synchAuto();
}
